use vlc::{Instance, Media, MediaPlayer};
use std::path::PathBuf;

use std::sync::{Arc, Mutex, Condvar};
use std::ffi::CString;

fn main()
    -> std::result::Result<(), ()>
{
    let args = std::env::args()
        .collect::<Vec<String>>();

    assert!(args.len() > 2);
    let filename = args[1].clone();
    let output = args[2].clone();
    let path = PathBuf::from(filename);

    let instance = Instance::new().unwrap();
    let media    = Media::new_path(&instance, &path).unwrap();
    let player   = MediaPlayer::new(&instance).unwrap();
    let evt_mgr  = player.event_manager();

    let raw_media = media.raw();
    let option_sout = CString::new(format!(concat!(
        ":sout=#transcode{{vcodec=h264,fps=60,width=720,height=640}}",
        ":std{{access=file,mux=mkv,dst={dstfile}}}"),
        dstfile = &output)).unwrap();
    let option_start = CString::new(":start-time=5").unwrap();
    let option_stop  = CString::new(":stop-time=10").unwrap();

    unsafe {
        vlc::sys::libvlc_media_add_option(raw_media, option_sout.as_ptr());
        vlc::sys::libvlc_media_add_option(raw_media, option_start.as_ptr());
        vlc::sys::libvlc_media_add_option(raw_media, option_stop.as_ptr());
    }


    let local_var  = Arc::new((Mutex::new(true), Condvar::new()));
    let libvlc_var = local_var.clone();

    evt_mgr.attach(vlc::EventType::MediaPlayerEndReached, move |event, obj| {
        println!("MediaPlayer: EndReached");

        let (lock, cond_var) = &*libvlc_var;
        let mut running = lock.lock().unwrap();
        *running = false;
        cond_var.notify_one();
    }).unwrap();

    player.set_media(&media);
    player.play().unwrap();

    let (lock, cond_var) = &*local_var;
    let mut running = lock.lock().unwrap();

    while *running
    {
        running = cond_var.wait(running).unwrap();
    }

    println!(" - Transcode just finished");

    Ok(())
}
